/*
 * signals/catc.c --
 *
 *	This program demonstrates the C library functions for handling
 *	signals. Note that signal handlers need to be reinstalled and
 *	that signals can interrupt I/O system calls.
 */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

static void
handler(int signum)
{
    if (signum == SIGINT) {
        fprintf(stderr, "catc: handler: Interrupt\n");
    } else {
        fprintf(stderr, "catc: handler: %d\n", signum);
    }
    if (signal(SIGINT, handler) == SIG_ERR) {
	perror("signal");
	exit(EXIT_FAILURE);
    }
}

int
main(void)
{
    char c;

    if (signal(SIGINT, handler) == SIG_ERR) {
        perror("signal");
        return EXIT_FAILURE;
    }

    while (! feof(stdin)) {
        while ((c = getc(stdin)) != EOF) {
            (void) putc(c, stdout);
        }
        if (ferror(stdin)) {
            if (errno == EINTR) {
                clearerr(stdin);
                continue;
            }
            break;
        }
    }

    if (ferror(stdin) || fflush(stdout) == EOF) {
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
